@@ -6,6 +6,7 @@ import lxml.html
 import htmllaundry
 from rtfng.Elements import Document
 from rtfng.Elements import StyleSheet
+from rtfng.document.base import RawCode
 from rtfng.document.character import TEXT
 from rtfng.document.paragraph import Paragraph
 from rtfng.document.paragraph import Cell
@@ -43,6 +44,10 @@ PRIORITY_NAMES = {
 }
 
 
+# These are coded incorrectly in pyrtf-ng: the field name must be all caps
+PAGE_NUMBER   = RawCode(r'{\field{\fldinst PAGE}}')
+TOTAL_PAGES   = RawCode(r'{\field{\fldinst NUMPAGES}}')
+
 def createDocument(survey_session):
     from rtfng.Styles import TextStyle
     from rtfng.Styles import ParagraphStyle
@@ -70,6 +75,11 @@ def createDocument(survey_session):
     stylesheet.ParagraphStyles.append(ParagraphStyle(
         "Footer",
         style.Copy(), ParagraphPropertySet()))
+    pagenumber_style = ParagraphStyle("PageNumber", style.Copy())
+    pagenumber_style.SetBasedOn(stylesheet.ParagraphStyles.Footer)
+    pagenumber_style.SetParagraphPropertySet(
+            ParagraphPropertySet(alignment=ParagraphPropertySet.RIGHT))
+    stylesheet.ParagraphStyles.append(pagenumber_style)
 
     style.textProps.italic = False
     style.textProps.size = 36
@@ -186,9 +196,21 @@ def createSection(document, survey, survey_session, request):
     footer = Paragraph(
         document.StyleSheet.ParagraphStyles.Footer,
         "".join(["\u%s?" % str(ord(e)) for e in footer]))
+
     section = Section()
+    page_header = []
+    for part in t(_(u'Page ${number} of ${total}')).split():
+        if part == '${number}':
+            page_header.append(PAGE_NUMBER)
+        elif part == '${total}':
+            page_header.append(TOTAL_PAGES)
+        else:
+            page_header.append(part)
+        page_header.append(' ')
     section.Header.append(Paragraph(
         document.StyleSheet.ParagraphStyles.Footer, survey_session.title))
+    section.Header.append(Paragraph(
+        document.StyleSheet.ParagraphStyles.PageNumber, *page_header))
     section.Footer.append(footer)
     section.SetBreakType(section.PAGE)
     document.Sections.append(section)
@@ -865,18 +887,23 @@ class ActionPlanTimeline(grok.View):
         """Find all data that should be included in the report.
 
         The data is returned as a list of tuples containing a
+        :py:class:`Module <euphorie.client.model.Module>`,
         :py:class:`Risk <euphorie.client.model.Risk>` and
         :py:class:`ActionPlan <euphorie.client.model.ActionPlan>`. Each
         entry in the list will correspond to a row in the generated Excel
         file.
         """
-        query = Session.query(model.SurveyTreeItem, model.ActionPlan)\
-            .outerjoin(model.ActionPlan)\
-            .filter(model.SurveyTreeItem.type == 'risk')\
-            .filter(model.SurveyTreeItem.session == self.session)\
+        query = Session.query(model.Module, model.Risk, model.ActionPlan)\
+            .filter(sql.and_(model.Module.depth == 1,
+                             model.Module.session == self.session))\
             .filter(sql.not_(model.SKIPPED_PARENTS))\
             .filter(sql.or_(model.MODULE_WITH_RISK_OR_TOP5_FILTER,
                             model.RISK_PRESENT_OR_TOP5_FILTER))\
+            .join((model.Risk,
+                   sql.and_(model.Risk.path.startswith(model.Module.path),
+                            model.Risk.session == self.session)))\
+            .outerjoin((model.ActionPlan,
+                   model.ActionPlan.risk_id == model.Risk.id))\
             .order_by(model.ActionPlan.planning_start,
                       model.SurveyTreeItem.path)
         return query.all()
@@ -907,6 +934,8 @@ class ActionPlanTimeline(grok.View):
                 default=u'Who is responsible?')),
         ('measure', 'budget',
             _('label_action_plan_budget', default=u'Budget (in Euro)')),
+        ('module', 'title',
+            _('label_section', default=u'Section')),
         ('risk', 'number',
             _('label_risk_number', default=u'Risk number')),
         ('risk', 'title',
@@ -935,7 +964,9 @@ class ActionPlanTimeline(grok.View):
         for (column, (type, key, title)) in enumerate(self.columns):
             sheet.cell(row=0, column=column).value = t(title)
 
-        for (row, (risk, measure)) in enumerate(self.get_measures(), 1):
+        for (row, (module, risk, measure)) in \
+                enumerate(self.get_measures(), 1):
+
             column = 0
             zodb_node = survey.restrictedTraverse(risk.zodb_path.split('/'))
             for (type, key, title) in self.columns:
@@ -950,6 +981,9 @@ class ActionPlanTimeline(grok.View):
                         if zodb_node.problem_description and \
                                 zodb_node.problem_description.strip():
                             value = zodb_node.problem_description
+                elif type == 'module':
+                    value = getattr(module, key, None)
+
                 if value is not None:
                     sheet.cell(row=row, column=column).value = value
                 column += 1
